<?php

namespace Drupal\monitoring_ntfy\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\monitoring\Entity\SensorResultDataInterface;

/**
 * Provide the configuration form for the NtfyService.
 */
class MonitoringNtfySettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'monitoring_ntfy_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['monitoring_ntfy.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('monitoring_ntfy.settings');

    // Configure the ntfy.sh server instance.
    $form['ntfy_server'] = [
      '#type' => 'url',
      '#title' => $this->t('NTFY.sh server'),
      '#default_value' => $config->get('ntfy_server') ?? 'https://ntfy.sh',
      '#description' => $this->t('Enter the ntfy.sh url for sending notifications.'),
    ];

    // Configure the default topic for sending notifications.
    $form['ntfy_default_topic'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Notification Topic'),
      '#default_value' => $config->get('ntfy_default_topic'),
      '#description' => $this->t('Enter the default topic to send notifications to.'),
    ];

    // Allow the user to choose on which severities to send a notification.
    $severities = [
      SensorResultDataInterface::STATUS_OK => $this->t('OK'),
      SensorResultDataInterface::STATUS_UNKNOWN => $this->t('Unknown'),
      SensorResultDataInterface::STATUS_WARNING => $this->t('Warning'),
      SensorResultDataInterface::STATUS_CRITICAL => $this->t('Critical'),
      SensorResultDataInterface::STATUS_INFO => $this->t('Info'),
    ];
    $form['ntfy_severity_triggers'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Severity Triggers'),
      '#description' => $this->t('Select the severities which trigger a notification on change.'),
      '#options' => $severities,
      '#default_value' => $config->get('ntfy_severity_triggers') ?? [],
    ];

    // Allow for setting authentication details. Using environment variables
    // are recommended.
    $form['authentication'] = [
      '#type' => 'details',
      '#title' => $this->t('Authentication'),
      '#description' => $this->t('Enter authentication details if needed.'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];

    $env_auth_method = getenv('MONITORING_NTFY_AUTH_METHOD');

    $form['authentication']['method'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose the authentication method'),
      '#options' => [
        'token' => $this->t('Auth Token'),
        'user' => $this->t('Username and Password'),
      ],
      '#default_value' => is_string($env_auth_method) ? $config->get('method') : '',
      '#disabled' => is_string(getenv('MONITORING_NTFY_AUTH_METHOD')),
    ];

    $auth_form_fields = [];
    if ($env_auth_method === 'token') {
      $auth_form_fields = [
        'access_token' => [
          '#type' => 'password',
          '#title' => $this->t('Access Token'),
          '#description' => $this->t('Enter the provided access token.'),
        ],
      ];
    }
    if ($env_auth_method === 'user') {
      $auth_form_fields = [
        'user' => [
          '#type' => 'textfield',
          '#title' => $this->t('User Name'),
          '#description' => $this->t('Provide the user to use for sending notifications.'),
        ],
        'password' => [
          '#type' => 'password',
          '#title' => $this->t('Password'),
          '#description' => $this->t('Provide the password of the user.'),
        ],
      ];
    }

    foreach ($auth_form_fields as $config_key => $config_form_details) {
      $env_string = 'MONITORING_NTFY_' . strtoupper($config_key);
      if (is_string(getenv($env_string))) {
        $form['authentication'][$config_key] = [
          '#type' => 'textfield',
          '#title' => $config_form_details['#title'],
          '#description' => $config_form_details['#description'],
          '#default_value' => $this->t('This setting has been set with $_ENV["%env_variable_name"]', ["%env_variable_name" => $env_string]),
          '#disabled' => TRUE,
        ];
      }
      else {
        $form['authentication'][$config_key] = [
          '#type' => $config_form_details['#type'],
          '#title' => $config_form_details['#title'],
          '#description' => $config_form_details['#description'],
          '#default_value' => $config->get('ntfy_authentication_' . $config_key),
        ];
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    // Set ntfy settings.
    $this->config('monitoring_ntfy.settings')
      ->set('ntfy_server', $form_state->getValue('ntfy_server'))
      ->set('ntfy_default_topic', $form_state->getValue('ntfy_default_topic'))
      ->set('ntfy_severity_triggers', $form_state->getValue('ntfy_severity_triggers'))
      ->save();

    // Set auth details.
    $auth_keys = [
      'method',
      'access_token',
      'user',
      'password',
    ];
    foreach ($auth_keys as $auth_key) {
      if (is_string($form_state->getValue($auth_key))) {
        $this->config('monitoring_ntfy.settings')
          ->set('ntfy_authentication_' . $auth_key, $form_state->getValue($auth_key))
          ->save();
      }
    }
  }

}
